/****************************************************************************
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the Qt Mobility Components.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://qt.digia.com/licensing.  For further information
** use the contact form at http://qt.digia.com/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Digia gives you certain additional
** rights.  These rights are described in the Digia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "n900accelerometer.h"
#include "../util/util.h"
#include <QFile>
#include <QDebug>
#include <stdio.h>

char const * const n900accelerometer::id("n900.accelerometer");
char const * const n900accelerometer::filename("/sys/class/i2c-adapter/i2c-3/3-001d/coord");
char const * const n900accelerometer::range("/sys/class/i2c-adapter/i2c-3/3-001d/scale");
char const * const n900accelerometer::rate("/sys/class/i2c-adapter/i2c-3/3-001d/rate");

extern bool portraitOrientation;

n900accelerometer::n900accelerometer(QSensor *sensor)
    : n900filebasedsensor(sensor)
{
    setReading<QAccelerometerReading>(&m_reading);
    // According to the kernel driver, this runs at 100 or 400Hz
    // but we can't change the rate because we don't run as root.
    // The hardware seems to run at 100Hz by default so report that.
    // Report 1-100Hz so the app can poll less frequently if it wants to.
    addDataRate(1, 100);
    // According to the kernel driver this reports +- 2G or +- 8G (with lower accuracy)
    // but we can't change the range because we don't run as root.
    // The hardware seems to run at +- 2G by default so report that.
    addOutputRange(-22.418, 22.418, 0.17651); // 2G
    setDescription(QLatin1String("lis302dl"));
}

void n900accelerometer::start()
{
    if (!QFile::exists(QLatin1String(filename)))
        goto error;

    n900filebasedsensor::start();
    return;

error:
    sensorStopped();
}

void n900accelerometer::poll()
{
    // Note that this is a rather inefficient way to generate this data.
    // Ideally the kernel would scale the hardware's values to m/s^2 for us
    // and give us a timestamp along with that data.

    FILE *fd = fopen(filename, "r");
    if (!fd) return;
    int x, y, z;
    int rs = fscanf(fd, "%i %i %i", &x, &y, &z);
    fclose(fd);
    if (rs != 3) return;

    // Convert from milli-Gs to meters per second per second
    // Using 1 G = 9.80665 m/s^2
    qreal ax = x * 0.00980665;
    qreal ay = y * -0.00980665;
    qreal az = z * -0.00980665;

    m_reading.setTimestamp(getTimestamp());
    if (portraitOrientation) {
        m_reading.setX(ay);
        m_reading.setY(-ax);
    } else {
        m_reading.setX(ax);
        m_reading.setY(ay);
    }
    m_reading.setZ(az);

    newReadingAvailable();
}

