/****************************************************************************
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the Qt Mobility Components.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://qt.digia.com/licensing.  For further information
** use the contact form at http://qt.digia.com/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Digia gives you certain additional
** rights.  These rights are described in the Digia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtTest/QtTest>
#include <qsysteminfo.h>

QTM_USE_NAMESPACE

#define SHOWDEBUGS

QSemaphore MainThreadRunningSemaphore;
QSemaphore OtherThreadRunningSemaphore;

class ThreadBase : public QThread
{
    Q_OBJECT
public:

    void run() {
#ifdef SHOWDEBUGS
        qDebug()<<"Thread::run - create object in thread: "<<thread()->currentThreadId();
#endif//SHOWDEBUGS
        construct();
#if 0
        qDebug()<<"Thread::run - read something directly from object in this thread: "
                <<networkInfo.currentMode();
#endif//SHOWDEBUGS
        MainThreadRunningSemaphore.release();
    }

    virtual void construct() = 0;
};

class QSystemBatteryInfoThread : public ThreadBase
{
    void construct() {
        QSystemBatteryInfo info;
    }
};

class QSystemDeviceInfoThread : public ThreadBase
{
    void construct() {
        QSystemDeviceInfo info;
    }
};

class QSystemInfoThread : public ThreadBase
{
    void construct() {
        QSystemInfo info;
    }
};

class QSystemNetworkInfoThread : public ThreadBase
{
    void construct() {
        QSystemNetworkInfo info;
    }
};

class QSystemScreenSaverThread : public ThreadBase
{
    void construct() {
        QSystemScreenSaver info;
    }
};

class QSystemStorageInfoThread : public ThreadBase
{
    void construct() {
        QSystemStorageInfo info;
    }
};

class tst_QSystemInfo_CreateInfoOutsideMainThread : public QObject
{
    Q_OBJECT

private:
    void runThread(ThreadBase& testThread) {
        testThread.start();
        MainThreadRunningSemaphore.acquire();
        OtherThreadRunningSemaphore.release();
        testThread.wait();
    }

    void logThread(const QString message) {
#ifdef SHOWDEBUGS
        qDebug() << message << thread()->currentThreadId();
#endif//SHOWDEBUGS
    }

private Q_SLOTS:
    void testBatteryInfo() {
        logThread("running in thread:");
        QSystemBatteryInfoThread testThread;
        runThread(testThread);
    }

    void testDeviceInfo() {
        logThread("running in thread:");
        QSystemDeviceInfoThread testThread;
        runThread(testThread);
    }

    void testInfo() {
        logThread("running in thread:");
        QSystemInfoThread testThread;
        runThread(testThread);
    }

    void testNetworkInfo() {
        logThread("running in thread:");
        QSystemNetworkInfoThread testThread;
        runThread(testThread);
    }

    void testScreenSaver() {
        logThread("running in thread:");
        QSystemScreenSaverThread testThread;
        runThread(testThread);
    }

    void testStorageInfo() {
        logThread("running in thread:");
        QSystemStorageInfoThread testThread;
        runThread(testThread);
    }
};

QTEST_MAIN(tst_QSystemInfo_CreateInfoOutsideMainThread);

#include "tst_CreateInfoOutsideMainThread.moc"
